package kawigi.tools;
import kawigi.tools.fs.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

public class FloodGrapherGUI extends JFrame implements ActionListener, WindowListener
{
	public static void main(String args[])
	{
		FloodGrapherGUI frame;
		if (args.length > 0)
			frame = new FloodGrapherGUI(args[args.length-1]);
		else
			frame = new FloodGrapherGUI("kawigi" + File.separator + "resource" + File.separator + "floodgrapher.ini");
		frame.show();
	}
	
	private JTabbedPane tabs;
	private Vector formatList, guiPlugins;
	private File defaultDirectory, initFile;
	private boolean overwrite = true;
	private FloodScriptParser parser;
	private JMenuItem exitItem, optionsItem;
	private JDialog optionsDialog;
	private JComboBox plafBox;
	private JCheckBox saveSettingsBox;
	private JButton okButton, cancelButton;
	
	public FloodGrapherGUI(String initFileName)
	{
		super("Flood Grapher 2.0");
		addWindowListener(this);
		
		setIconImage(Toolkit.getDefaultToolkit().getImage("kawigi/resource/floodgrapher.gif"));
		JMenuBar menubar = new JMenuBar();
		JMenu fileMenu = new JMenu("File");
		menubar.add(fileMenu);
		setJMenuBar(menubar);
		
		initFile = new File(initFileName);
		if (!initFile.exists())
			createDefaultInitFile(initFile);
		readInitFile(initFile);
		
		tabs = new JTabbedPane();
		for (int i=0; i<guiPlugins.size(); i++)
		{
			FloodGrapherGUIPlugin plugin = (FloodGrapherGUIPlugin)guiPlugins.elementAt(i);
			tabs.add(plugin.getName(), plugin);
			plugin.initialize();
		}
		optionsItem = new JMenuItem("Options...");
		fileMenu.add(optionsItem);
		optionsItem.addActionListener(this);
		fileMenu.addSeparator();
		exitItem = new JMenuItem("Exit");
		fileMenu.add(exitItem);
		exitItem.addActionListener(this);
		optionsDialog = new JDialog(this, "Options", true);
		optionsDialog.getContentPane().setLayout(new BoxLayout(optionsDialog.getContentPane(), BoxLayout.Y_AXIS));
		JPanel plafPanel = new JPanel();
		plafPanel.setLayout(new BoxLayout(plafPanel, BoxLayout.X_AXIS));
		plafPanel.add(new JLabel("Look and Feel:"));
		plafBox = new JComboBox();
		UIManager.LookAndFeelInfo[] infos = UIManager.getInstalledLookAndFeels();
		for (int i=0; i<infos.length; i++)
		{
			plafBox.addItem(infos[i].getName());
		}
		plafBox.setSelectedItem(UIManager.getLookAndFeel().getName());
		plafPanel.add(plafBox);
		optionsDialog.getContentPane().add(plafPanel);
		saveSettingsBox = new JCheckBox("save settings", overwrite);
		optionsDialog.getContentPane().add(saveSettingsBox);
		JPanel okCancelPanel = new JPanel();
		okCancelPanel.setLayout(new BoxLayout(okCancelPanel, BoxLayout.X_AXIS));
		okButton = new JButton("OK");
		okButton.addActionListener(this);
		cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(this);
		okCancelPanel.add(okButton);
		okCancelPanel.add(cancelButton);
		optionsDialog.getContentPane().add(okCancelPanel);
		getContentPane().add(tabs);
		SwingUtilities.updateComponentTreeUI(this);
	}
	
	public Vector getFormatList()
	{
		return formatList;
	}
	
	public Vector getPlugins()
	{
		return guiPlugins;
	}
	
	public FloodScriptParser getFloodScriptParser()
	{
		return parser;
	}
	
	public JTabbedPane getTabbedPane()
	{
		return tabs;
	}
	
	public static void createDefaultInitFile(File file)
	{
		try
		{
			PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(file)));
			out.println("######################################################################");
			out.println("#     This is a configuration file for                               #");
			out.println("#          FloodGrapher 2.0 by Matthew Reeder (a.k.a. Kawigi)        #");
			out.println("#                                                                    #");
			out.println("#  It contains options for running the graphing utility, including   #");
			out.println("#  plugin support.  Use '#' signs for comments.  A different config  #");
			out.println("#  file may be used by specifying it on the command line, i.e. -     #");
			out.println("#       java kawigi.tools.FloodGrapher myInitFile.ini                #");
			out.println("#  This file is regularly changed by the program based on usage of   #");
			out.println("#  the program, unless the \"overwrite\" property is set to false.     #");
			out.println("######################################################################");
			out.println();
			out.println("# This property sets where the \"open file\" dialog starts:");
			out.println("defaultDirectory=[\".\"]");
			out.println();
			out.println("# Window properties when the program starts up:");
			out.println("LocationX=[i0]");
			out.println("LocationY=[i0]");
			out.println("SizeX=[i700]");
			out.println("SizeY=[i400]");
			out.println();
			out.println("# You can use this to load settings from another file as well.");
			out.println("# load other.ini");
			out.println();
			out.println("# You can set this to false if you don't want the program to change");
			out.println("# this initialization file.  Note that plugin settings also won't be");
			out.println("# saved if overwrite is set to false.");
			out.println("overwrite=[#t]");
			out.println();
			out.println("# Feel free to specify your favorite Pluggable Look and Feel, too:");
			out.println("plaf=[\"javax.swing.plaf.metal.MetalLookAndFeel\"]");
			out.println();
			out.println("# These are file format plugins for the grapher.  All the classes");
			out.println("# listed here should also implement the FileFormat interface and have");
			out.println("# an empty constructor.");
			out.println("fileformat=kawigi.tools.FloodMini14Format");
			out.println("fileformat=kawigi.tools.FloodGrapherFormat");
			out.println("fileformat=kawigi.tools.DummyFileFormat");
			out.println();
			out.println("# This part configures what functional plugins should be added to");
			out.println("# the FloodGrapher GUI.  They should all extend FloodGrapherGUIPlugin:");
			out.println();
			out.println("# I don't know why you wuoldn't want to have this show up:");
			out.println("plugin=kawigi.tools.graph.FloodGrapherPane(this)");
			out.println("# You can also set where the divider goes between the graph and the desktop");
			out.println("divider=[i395]");
			out.println();
			out.println("# The plugin configurator:");
			out.println("plugin=kawigi.tools.PluginManagerPanel(this)");
			out.println();
			out.println("# This is the FloodScript editor:");
			out.println("plugin=kawigi.tools.fs.FloodScriptEditorPane(this)");
			out.println();
			out.println("# This is to help you track codesize without running the command line:");
			out.println("plugin=kawigi.tools.codesize.CodesizePane(this)");
			out.println();
			out.println("# The help plugin:");
			out.println("plugin=kawigi.tools.help.HelpPanel([\"kawigi" + (File.separator.equals("\\") ? "\\\\" : File.separator) + "tools" + (File.separator.equals("\\") ? "\\\\" : File.separator) + "help" + (File.separator.equals("\\") ? "\\\\" : File.separator) + "\"])");
			out.close();
		}
		catch (IOException e)
		{
			System.err.println("Couldn't write init file: " + e);
		}
	}
	
	public void readInitFile(File file)
	{
		parser = new FloodScriptParser(this);
		parser.loadSettings(file);
		defaultDirectory = new File((String)parser.getObject("defaultDirectory", "."));
		int locationX = ((Number)parser.getObject("locationX", new Integer(0))).intValue();
		int locationY = ((Number)parser.getObject("locationX", new Integer(0))).intValue();
		int sizeX = ((Number)parser.getObject("sizeX", new Integer(700))).intValue();
		int sizeY = ((Number)parser.getObject("sizeY", new Integer(400))).intValue();
		setLocation(locationX, locationY);
		setSize(sizeX, sizeY);
		overwrite = ((Boolean)parser.getObject("overwrite", new Boolean(true))).booleanValue();
		Object fileformat = parser.getObject("fileFormat");
		formatList = parser.getObjectList("fileFormat");
		guiPlugins = parser.getObjectList("plugin");
		try
		{
			UIManager.setLookAndFeel((String)parser.getObject("plaf", UIManager.getLookAndFeel().getClass().getName()));
		}
		catch (Exception e)
		{
			System.err.println("Couldn't create specified plaf");
		}
	}
	
	public void saveSettings()
	{
		try
		{
			PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(initFile)));
			out.println("######################################################################");
			out.println("#     This is a configuration file for                               #");
			out.println("#          FloodGrapher 2.0 by Matthew Reeder (a.k.a. Kawigi)        #");
			out.println("#                                                                    #");
			out.println("#  It contains options for running the graphing utility, including   #");
			out.println("#  plugin support.  Use '#' signs for comments.  A different config  #");
			out.println("#  file may be used by specifying it on the command line, i.e. -     #");
			out.println("#       java kawigi.tools.FloodGrapher myInitFile.ini                #");
			out.println("#  This file is regularly changed by the program based on usage of   #");
			out.println("#  the program, unless the \"overwrite\" property is set to false.     #");
			out.println("#  See the documentation on FloodScript for more information         #");
			out.println("######################################################################");
			out.println();
			out.println("# This property sets where the \"open file\" dialog starts:");
			out.println("defaultDirectory=[\"" + defaultDirectory + "\"]");
			out.println();
			out.println("# Window properties when the program starts up:");
			out.println("LocationX=[i" + getX() + "]");
			out.println("LocationY=[i" + getY() + "]");
			out.println("SizeX=[i" + getWidth() + "]");
			out.println("SizeY=[i" + getHeight() + "]");
			out.println();
			out.println("# You can use this to load settings from another file as well.");
			out.println("# load other.ini");
			out.println();
			out.println("# You can set this to false if you don't want the program to change");
			out.println("# this initialization file.  Note that plugin settings also won't be");
			out.println("# saved if overwrite is set to false.");
			out.println("overwrite = " + (overwrite ? "[#t]" : "[#f]"));
			out.println();
			out.println("# Feel free to specify your favorite Pluggable Look and Feel, too:");
			out.println("plaf=[\"" + UIManager.getLookAndFeel().getClass().getName() + "\"]");
			out.println();
			out.println("# These are file format plugins for the grapher.  All the classes");
			out.println("# listed here should also implement the FileFormat interface and have");
			out.println("# an empty constructor.  If they don't have an empty constructor,");
			out.println("# you'll have to hack this file for them to work.");
			for (int i=0; i<formatList.size(); i++)
				out.println("fileformat=" + formatList.elementAt(i).getClass().getName());
			out.println();
			out.println("# This part configures what functional plugins should be added to");
			out.println("# the FloodGrapher GUI.  They should all extend FloodGrapherGUIPlugin:\n");
			for (int i=0; i<guiPlugins.size(); i++)
				out.println(((FloodGrapherGUIPlugin)guiPlugins.elementAt(i)).getFloodScriptCommand());
			out.close();
		}
		catch (IOException e)
		{
			System.err.println("Couldn't write init file: " + e);
		}
	}
	
	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == exitItem)
		{
			if (overwrite)
				saveSettings();
			dispose();
			System.exit(0);
		}
		else if (e.getSource() == optionsItem)
		{
			optionsDialog.pack();
			optionsDialog.show();
		}
		else if (e.getSource() == okButton)
		{
			optionsDialog.hide();
			overwrite = saveSettingsBox.isSelected();
			if (!plafBox.getSelectedItem().equals(UIManager.getLookAndFeel().getName()))
			{
				UIManager.LookAndFeelInfo[] infos = UIManager.getInstalledLookAndFeels();
				for (int i=0; i<infos.length; i++)
				{
					if (plafBox.getSelectedItem().equals(infos[i].getName()))
					{
						try
						{
							UIManager.setLookAndFeel(infos[i].getClassName());
							SwingUtilities.updateComponentTreeUI(this);
							return;
						}
						catch (Exception ex)
						{
							
							JOptionPane.showMessageDialog(this, "Could not load look and feel class.", "Error", JOptionPane.ERROR_MESSAGE);
						}
					}
				}
			}
		}
		else if (e.getSource() == cancelButton)
		{
			optionsDialog.hide();
			saveSettingsBox.setSelected(overwrite);
			plafBox.setSelectedItem(UIManager.getLookAndFeel().getName());
		}
	}
	
	public void windowClosing(WindowEvent e)
	{
		if (overwrite)
			saveSettings();
		dispose();
		System.exit(0);
	}

	public void windowActivated(WindowEvent e){}
	public void windowClosed(WindowEvent e){}
	public void windowDeactivated(WindowEvent e){}
	public void windowDeiconified(WindowEvent e){}
	public void windowIconified(WindowEvent e){}
	public void windowOpened(WindowEvent e){}
}
