package kawigi.tools.fs;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import java.awt.Color;

public class FloodScriptLexer implements DocumentListener, Runnable, ListModel, ListSelectionListener
{
	private int index, lineoffset, lines;
	private HashSet varnames;
	private Vector listlisteners;
	private MutableAttributeSet plain, variable, comment, literal, keyword, classes, operator;
	private DefaultStyledDocument doc;
	private JList lineNumbers;
	private JTextPane textPane;
	private Thread updater;
	private boolean needsUpdate, unsaved;
	
	public FloodScriptLexer(DefaultStyledDocument doc, JList lineNumbers, JTextPane textPane)
	{
		unsaved = false;
		this.doc = doc;
		this.lineNumbers = lineNumbers;
		this.textPane = textPane;
		listlisteners = new Vector();
		doc.addDocumentListener(this);
		lineNumbers.setModel(this);
		lineNumbers.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		lineNumbers.addListSelectionListener(this);
		needsUpdate = true;
		updater = new Thread(this);
		
		plain = new SimpleAttributeSet();
		StyleConstants.setFontFamily(plain, "Monospaced");
		StyleConstants.setFontSize(plain, 12);
		StyleConstants.setForeground(plain, Color.black);
		variable = new SimpleAttributeSet();
		StyleConstants.setFontFamily(variable, "Monospaced");
		StyleConstants.setFontSize(variable, 12);
		StyleConstants.setForeground(variable, new Color(0, 0, 128));
		comment = new SimpleAttributeSet();
		StyleConstants.setFontFamily(comment, "Monospaced");
		StyleConstants.setFontSize(comment, 12);
		StyleConstants.setItalic(comment, true);
		StyleConstants.setForeground(comment, new Color(0, 128, 0));
		literal = new SimpleAttributeSet();
		StyleConstants.setFontFamily(literal, "Monospaced");
		StyleConstants.setFontSize(literal, 12);
		StyleConstants.setForeground(literal, new Color(128, 0, 0));
		keyword = new SimpleAttributeSet();
		StyleConstants.setFontFamily(keyword, "Monospaced");
		StyleConstants.setFontSize(keyword, 12);
		StyleConstants.setForeground(keyword, Color.blue);
		classes = new SimpleAttributeSet();
		StyleConstants.setFontFamily(classes, "Monospaced");
		StyleConstants.setFontSize(classes, 12);
		StyleConstants.setForeground(classes, new Color(128, 0, 128));
		operator = new SimpleAttributeSet();
		StyleConstants.setFontFamily(operator, "Monospaced");
		StyleConstants.setFontSize(operator, 12);
		StyleConstants.setForeground(operator, new Color(255, 0, 0));
		
		updater.start();
	}
	
	public void changedUpdate(DocumentEvent e)
	{
		unsaved = true;
		needsUpdate = true;
	}
	
	public void insertUpdate(DocumentEvent e)
	{
		unsaved = true;
		needsUpdate = true;
	}
	
	public void removeUpdate(DocumentEvent e)
	{
		unsaved = true;
		needsUpdate = true;
	}
	
	public void notifySave()
	{
		unsaved = false;
	}
	
	public boolean needsSave()
	{
		return unsaved;
	}
	
	public synchronized void run()
	{
		while (Thread.currentThread() == updater)
		{
			if (needsUpdate)
			{
				try
				{
					parseText();
					needsUpdate = false;
				}
				catch (Exception e)
				{
					System.out.println(e);
					e.printStackTrace();
				}
			}
			try
			{
				wait(200);
			}
			catch (InterruptedException e)
			{
				break;
			}
		}
	}
	
	private String pad(int i)
	{
		String s = Integer.toString(i);
		while (s.length() < 4)
			s = " " + s;
		return s;
	}
	
	public void parseText()
	{
		if (lines != getSize())
		{
			lines = getSize();
			for (int i=0; i<listlisteners.size(); i++)
				((ListDataListener)listlisteners.elementAt(i)).contentsChanged(new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, Math.min(lines, getSize()), Math.max(lines, getSize())));
		}
		try
		{
			setAttributes(0, doc.getLength(), plain, true);
			String text = doc.getText(0, doc.getLength());
			//tokenizes on lines and also returns the delimiters so I can have
			//the right offsets:
			StringTokenizer tok = new StringTokenizer(text, "\n\r", true);
			lineoffset = 0;
			varnames = new HashSet();
			while (tok.hasMoreTokens())
			{
				index = 0;
				String line = tok.nextToken();
				parseLine(line);
				lineoffset += line.length();
			}
		}
		catch (BadLocationException e)
		{
			System.out.println(e);
		}
	}
	
	private void skipWhiteSpace(String line)
	{
		while (index < line.length() && Character.isWhitespace(line.charAt(index)))
			index++;
	}
	
	private void parseLine(String line)
	{
		skipWhiteSpace(line);
		if (index >= line.length())
			return;
		if (line.charAt(index) == '#')
		{
			setAttributes(lineoffset+index, line.length()-index, comment, true);
			return;
		}
		int endindex = index;
		while (endindex < line.length() && (Character.isLetterOrDigit(line.charAt(endindex)) || line.charAt(endindex) == '_'))
			endindex++;
		
		String varName = line.substring(index, endindex);
		if (varName.toLowerCase().equals("load") || varName.toLowerCase().equals("print"))
		{
			setAttributes(lineoffset+index, endindex-index, keyword, true);
			index = endindex;
		}
		else
		{
			varnames.add(varName);
			setAttributes(lineoffset+index, endindex-index, variable, true);
			index = endindex;
			skipWhiteSpace(line);
			if (index < line.length() && line.charAt(index) == '=')
			{
				setAttributes(lineoffset+index, 1, operator, true);
				index++;
			}
		}
		skipWhiteSpace(line);
		if (index < line.length())
			parseValue(line);
	}
	
	private void parseValue(String line)
	{
		if (index >= line.length())
			return;
		if (line.charAt(index) == '[')
		{
			int endindex = line.indexOf(']', index);
			if (endindex < 0)
				endindex = line.length()-1;
			setAttributes(lineoffset+index, endindex-index+1, literal, true);
			index = endindex+1;
		}
		else
		{
			int endindex = index;
			while (endindex < line.length() && (Character.isLetterOrDigit(line.charAt(endindex)) || line.charAt(endindex) == '_' || line.charAt(endindex) == '.' || line.charAt(endindex) == '$'))
				endindex++;
			String className = line.substring(index, endindex);
			if (className.equals("this") || className.equals("null"))
			{
				setAttributes(lineoffset+index, endindex-index, keyword, true);
				index = endindex;
			}
			else if (varnames.contains(className))
			{
				setAttributes(lineoffset+index, endindex-index, variable, true);
				index = endindex;
			}
			else
			{
				if (isClass(className))
				 	setAttributes(lineoffset+index, endindex-index, classes, true);
				index = endindex;
				skipWhiteSpace(line);
				if (index < line.length() && line.charAt(index) == '(')
				{
					do
					{
						setAttributes(lineoffset+index, 1, operator, true);
						index++;
						skipWhiteSpace(line);
						parseValue(line);
						skipWhiteSpace(line);
					}
					while (index < line.length() && line.charAt(index) == ',');
					if (index < line.length() && line.charAt(index) == ')')
						setAttributes(lineoffset+index, 1, operator, true);
				}
			}
		}
	}
	
	private static boolean isClass(String str)
	{
		try
		{
			Class.forName(str);
			return true;
		}
		catch (Exception e)
		{
			return false;
		}
	}
	
	public void setAttributes(int offset, int length, AttributeSet attrs, boolean replace)
	{
		try
		{
			doc.replace(offset, length, doc.getText(offset, length), attrs);
		}
		catch (BadLocationException e)
		{
			System.out.println(e);
		}
	}
	
	public void addListDataListener(ListDataListener l)
	{
		listlisteners.add(l);
	}
	
	public Object getElementAt(int index)
	{
		return pad(index+1) + ": ";
	}
	
	public int getSize()
	{
		return doc.getDefaultRootElement().getElementCount();
	}
	
	public void removeListDataListener(ListDataListener l)
	{
		listlisteners.remove(l);
	}
	
	public void valueChanged(ListSelectionEvent e)
	{
		int startLine = lineNumbers.getMinSelectionIndex();
		int endLine = lineNumbers.getMaxSelectionIndex();
		int position=0;
		int i;
		for (i=0; i<startLine; i++)
			System.out.println(position = doc.getParagraphElement(position).getEndOffset());
		int startIndex = doc.getParagraphElement(position).getStartOffset();
		for (; i<endLine; i++)
			System.out.println("2 " + (position = doc.getParagraphElement(position).getEndOffset()));
		int endIndex = doc.getParagraphElement(position).getEndOffset();
		textPane.select(startIndex, endIndex);
		System.out.println(startIndex + " " + endIndex);
		if (!e.getValueIsAdjusting())
		{
			System.out.println("Focus");
			textPane.requestFocus();
		}
		else
			System.out.println("no focus for you!!");
	}
}
